@@ -16,12 +16,15 @@ from typing import (
     Counter,
     Dict,
     Iterable,
+    Iterator,
     List,
     Optional,
     Sequence,
     Set,
     Tuple,
+    TypeVar,
     Union,
+    cast,
 )
 
 import attr
@@ -53,22 +56,30 @@ from swh.model.swhids import CoreSWHID, ExtendedObjectType, ExtendedSWHID
 from swh.model.swhids import ObjectType as SwhidObjectType
 from swh.storage.interface import (
     VISIT_STATUSES,
+    HashDict,
     ListOrder,
     OriginVisitWithStatuses,
     PagedResult,
     PartialBranches,
     Sha1,
+    TotalHashDict,
 )
 from swh.storage.objstorage import ObjStorage
 from swh.storage.utils import map_optional, now
 from swh.storage.writer import JournalWriter
 
 from . import converters
-from ..exc import HashCollision, StorageArgumentException
+from ..exc import (
+    HashCollision,
+    StorageArgumentException,
+    UnknownMetadataAuthority,
+    UnknownMetadataFetcher,
+)
 from ..utils import remove_keys
 from .common import TOKEN_BEGIN, TOKEN_END, hash_url
 from .cql import CqlRunner
 from .model import (
+    BaseRow,
     ContentRow,
     DirectoryEntryRow,
     DirectoryRow,
@@ -82,6 +93,7 @@ from .model import (
     RawExtrinsicMetadataByIdRow,
     RawExtrinsicMetadataRow,
     RevisionParentRow,
+    RevisionRow,
     SkippedContentRow,
     SnapshotBranchRow,
     SnapshotRow,
@@ -94,6 +106,58 @@ BULK_BLOCK_CONTENT_LEN_MAX = 10000
 DIRECTORY_ENTRIES_INSERT_ALGOS = ["one-by-one", "concurrent", "batch"]
 
 
+TResult = TypeVar("TResult")
+TRow = TypeVar("TRow", bound=BaseRow)
+
+
+def _get_paginated_sha1_partition(
+    partition_id: int,
+    nb_partitions: int,
+    page_token: Optional[str],
+    limit: int,
+    get_range: Callable[[int, int, int], Iterator[Tuple[int, TRow]]],
+    convert: Callable[[TRow], TResult],
+    skip_row: Callable[[TRow], bool] = lambda row: False,
+) -> PagedResult[TResult]:
+    """Implements the bulk of ``content_get_partition``, ``directory_get_partition``,
+    ...:
+
+    1. computes range bounds
+    2. applies pagination token
+    3. converts to final objects
+    4. extracts next pagination token
+    """
+    if limit is None:
+        raise StorageArgumentException("limit should not be None")
+
+    # Compute start and end of the range of tokens covered by the
+    # requested partition
+    partition_size = (TOKEN_END - TOKEN_BEGIN) // nb_partitions
+    range_start = TOKEN_BEGIN + partition_id * partition_size
+    range_end = TOKEN_BEGIN + (partition_id + 1) * partition_size
+
+    # offset the range start according to the `page_token`.
+    if page_token is not None:
+        if not (range_start <= int(page_token) <= range_end):
+            raise StorageArgumentException("Invalid page_token.")
+        range_start = int(page_token)
+
+    next_page_token: Optional[str] = None
+
+    rows = get_range(range_start, range_end, limit + 1)
+    results = []
+    for counter, (tok, row) in enumerate(rows):
+        if skip_row(row):
+            continue
+        if counter >= limit:
+            next_page_token = str(tok)
+            break
+        results.append(convert(row))
+
+    assert len(results) <= limit
+    return PagedResult(results=results, next_page_token=next_page_token)
+
+
 class CassandraStorage:
     def __init__(
         self,
@@ -136,7 +200,7 @@ class CassandraStorage:
         self._consistency_level = consistency_level
         self._set_cql_runner()
         self.journal_writer: JournalWriter = JournalWriter(journal_writer)
-        self.objstorage: ObjStorage = ObjStorage(objstorage)
+        self.objstorage: ObjStorage = ObjStorage(self, objstorage)
         self._allow_overwrite = allow_overwrite
 
         if directory_entries_insert_algo not in DIRECTORY_ENTRIES_INSERT_ALGOS:
@@ -146,6 +210,18 @@ class CassandraStorage:
             )
         self._directory_entries_insert_algo = directory_entries_insert_algo
 
+    @property
+    def hosts(self) -> List[str]:
+        return self._hosts
+
+    @property
+    def keyspace(self) -> str:
+        return self._keyspace
+
+    @property
+    def port(self) -> int:
+        return self._port
+
     def _set_cql_runner(self):
         """Used by tests when they need to reset the CqlRunner"""
         self._cql_runner: CqlRunner = CqlRunner(
@@ -157,6 +233,10 @@ class CassandraStorage:
 
         return True
 
+    ##########################
+    # Content
+    ##########################
+
     def _content_get_from_hashes(self, algo, hashes: List[bytes]) -> Iterable:
         """From the name of a hash algorithm and a value of that hash,
         looks up the "hash -> token" secondary table (content_by_{algo})
@@ -272,7 +352,7 @@ class CassandraStorage:
     def content_add_metadata(self, content: List[Content]) -> Dict[str, int]:
         return self._content_add(content, with_data=False)
 
-    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+    def content_get_data(self, content: Union[Sha1, HashDict]) -> Optional[bytes]:
         # FIXME: Make this method support slicing the `data`
         return self.objstorage.content_get(content)
 
@@ -283,39 +363,23 @@ class CassandraStorage:
         page_token: Optional[str] = None,
         limit: int = 1000,
     ) -> PagedResult[Content]:
-        if limit is None:
-            raise StorageArgumentException("limit should not be None")
-
-        # Compute start and end of the range of tokens covered by the
-        # requested partition
-        partition_size = (TOKEN_END - TOKEN_BEGIN) // nb_partitions
-        range_start = TOKEN_BEGIN + partition_id * partition_size
-        range_end = TOKEN_BEGIN + (partition_id + 1) * partition_size
-
-        # offset the range start according to the `page_token`.
-        if page_token is not None:
-            if not (range_start <= int(page_token) <= range_end):
-                raise StorageArgumentException("Invalid page_token.")
-            range_start = int(page_token)
-
-        next_page_token: Optional[str] = None
-
-        rows = self._cql_runner.content_get_token_range(
-            range_start, range_end, limit + 1
-        )
-        contents = []
-        for counter, (tok, row) in enumerate(rows):
-            if row.status == "absent":
-                continue
+        def convert(row: ContentRow) -> Content:
             row_d = row.to_dict()
-            if counter >= limit:
-                next_page_token = str(tok)
-                break
             row_d.pop("ctime")
-            contents.append(Content(**row_d))
+            return Content(**row_d)
 
-        assert len(contents) <= limit
-        return PagedResult(results=contents, next_page_token=next_page_token)
+        def is_absent(row: ContentRow) -> bool:
+            return row.status == "absent"
+
+        return _get_paginated_sha1_partition(
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            self._cql_runner.content_get_token_range,
+            convert,
+            skip_row=is_absent,
+        )
 
     def content_get(
         self, contents: List[bytes], algo: str = "sha1"
@@ -336,10 +400,10 @@ class CassandraStorage:
             contents_by_hash[key(content)] = content
         return [contents_by_hash.get(hash_) for hash_ in contents]
 
-    def content_find(self, content: Dict[str, Any]) -> List[Content]:
+    def content_find(self, content: HashDict) -> List[Content]:
         return self._content_find_many([content])
 
-    def _content_find_many(self, contents: List[Dict[str, Any]]) -> List[Content]:
+    def _content_find_many(self, contents: List[HashDict]) -> List[Content]:
         # Find an algorithm that is common to all the requested contents.
         # It will be used to do an initial filtering efficiently.
         # TODO: prioritize sha256, we can do more efficient lookups from this hash.
@@ -355,14 +419,16 @@ class CassandraStorage:
 
         results = []
         rows = self._content_get_from_hashes(
-            common_algo, [content[common_algo] for content in contents]
+            common_algo,
+            [content[common_algo] for content in cast(List[dict], contents)],
         )
         for row in rows:
             # Re-check all the hashes, in case of collisions (either of the
             # hash of the partition key, or the hashes in it)
             for content in contents:
                 for algo in HASH_ALGORITHMS:
-                    if content.get(algo) and getattr(row, algo) != content[algo]:
+                    hash_ = content.get(algo)
+                    if hash_ and getattr(row, algo) != hash_:
                         # This hash didn't match; discard the row.
                         break
                 else:
@@ -377,15 +443,15 @@ class CassandraStorage:
         return results
 
     def content_missing(
-        self, contents: List[Dict[str, Any]], key_hash: str = "sha1"
+        self, contents: List[HashDict], key_hash: str = "sha1"
     ) -> Iterable[bytes]:
         if key_hash not in DEFAULT_ALGORITHMS:
             raise StorageArgumentException(
                 "key_hash should be one of {','.join(DEFAULT_ALGORITHMS)}"
             )
 
-        contents_with_all_hashes = []
-        contents_with_missing_hashes = []
+        contents_with_all_hashes: List[TotalHashDict] = []
+        contents_with_missing_hashes: List[HashDict] = []
         for content in contents:
             if DEFAULT_ALGORITHMS <= set(content):
                 contents_with_all_hashes.append(content)
@@ -396,7 +462,7 @@ class CassandraStorage:
         for content in self._cql_runner.content_missing_from_all_hashes(
             contents_with_all_hashes
         ):
-            yield content[key_hash]
+            yield content[key_hash]  # type: ignore
 
         if contents_with_missing_hashes:
             # For these, we need the expensive index lookups + main table.
@@ -426,7 +492,7 @@ class CassandraStorage:
                 # missing_content. (its length is at most 1, unless there is a
                 # collision)
                 found_contents_with_same_hash = found_contents_by_hash[algo][
-                    missing_content[algo]
+                    missing_content[algo]  # type: ignore
                 ]
 
                 # Check if there is a found_content that matches all hashes in the
@@ -444,7 +510,7 @@ class CassandraStorage:
                         break
                 else:
                     # Not found
-                    yield missing_content[key_hash]
+                    yield missing_content[key_hash]  # type: ignore
 
     def content_missing_per_sha1(self, contents: List[bytes]) -> Iterable[bytes]:
         return self.content_missing([{"sha1": c} for c in contents])
@@ -461,6 +527,10 @@ class CassandraStorage:
         assert content, "Could not find any content"
         return content.sha1_git
 
+    ##########################
+    # SkippedContent
+    ##########################
+
     def _skipped_content_add(self, contents: List[SkippedContent]) -> Dict[str, int]:
         # Filter-out content already in the database.
         if not self._allow_overwrite:
@@ -498,6 +568,10 @@ class CassandraStorage:
             if not self._cql_runner.skipped_content_get_from_pk(content):
                 yield {algo: content[algo] for algo in DEFAULT_ALGORITHMS}
 
+    ##########################
+    # Directory
+    ##########################
+
     def directory_add(self, directories: List[Directory]) -> Dict[str, int]:
         to_add = {d.id: d for d in directories}.values()
         if not self._allow_overwrite:
@@ -681,6 +755,26 @@ class CassandraStorage:
         assert directory, "Could not find any directory"
         return directory.id
 
+    def directory_get_id_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Sha1Git]:
+        return _get_paginated_sha1_partition(
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            self._cql_runner.directory_get_token_range,
+            operator.attrgetter("id"),
+        )
+
+    ##########################
+    # Revision
+    ##########################
+
     def revision_add(self, revisions: List[Revision]) -> Dict[str, int]:
         # Filter-out revisions already in the database
         if not self._allow_overwrite:
@@ -780,6 +874,27 @@ class CassandraStorage:
                 yield rev.to_dict()
                 yield from self._get_parent_revs(parents, seen, limit, short)
 
+    def revision_get_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Revision]:
+        def convert(row: RevisionRow) -> Revision:
+            return converters.revision_from_db(
+                row, parents=tuple(self._cql_runner.revision_parent_get(row.id))
+            )
+
+        return _get_paginated_sha1_partition(
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            self._cql_runner.revision_get_token_range,
+            convert,
+        )
+
     def revision_log(
         self,
         revisions: List[Sha1Git],
@@ -787,19 +902,29 @@ class CassandraStorage:
         limit: Optional[int] = None,
     ) -> Iterable[Optional[Dict[str, Any]]]:
         seen: Set[Sha1Git] = set()
-        yield from self._get_parent_revs(revisions, seen, limit, False)
+        yield from cast(
+            Iterable[Optional[Dict[str, Any]]],
+            self._get_parent_revs(revisions, seen, limit, short=False),
+        )
 
     def revision_shortlog(
         self, revisions: List[Sha1Git], limit: Optional[int] = None
     ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
         seen: Set[Sha1Git] = set()
-        yield from self._get_parent_revs(revisions, seen, limit, True)
+        yield from cast(
+            Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]],
+            self._get_parent_revs(revisions, seen, limit, short=True),
+        )
 
     def revision_get_random(self) -> Sha1Git:
         revision = self._cql_runner.revision_get_random()
         assert revision, "Could not find any revision"
         return revision.id
 
+    ##########################
+    # Release
+    ##########################
+
     def release_add(self, releases: List[Release]) -> Dict[str, int]:
         if not self._allow_overwrite:
             to_add = {r.id: r for r in releases}.values()
@@ -827,11 +952,31 @@ class CassandraStorage:
 
         return [rels.get(rel_id) for rel_id in releases]
 
+    def release_get_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Release]:
+        return _get_paginated_sha1_partition(
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            self._cql_runner.release_get_token_range,
+            lambda row: converters.release_from_db(row),
+        )
+
     def release_get_random(self) -> Sha1Git:
         release = self._cql_runner.release_get_random()
         assert release, "Could not find any release"
         return release.id
 
+    ##########################
+    # Snapshot
+    ##########################
+
     def snapshot_add(self, snapshots: List[Snapshot]) -> Dict[str, int]:
         if not self._allow_overwrite:
             to_add = {s.id: s for s in snapshots}.values()
@@ -881,6 +1026,22 @@ class CassandraStorage:
             "next_branch": d["next_branch"],
         }
 
+    def snapshot_get_id_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Sha1Git]:
+        return _get_paginated_sha1_partition(
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            self._cql_runner.snapshot_get_token_range,
+            operator.attrgetter("id"),
+        )
+
     def snapshot_count_branches(
         self,
         snapshot_id: Sha1Git,
@@ -975,35 +1136,9 @@ class CassandraStorage:
         assert snapshot, "Could not find any snapshot"
         return snapshot.id
 
-    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
-        results: Dict[Sha1Git, List[Dict]] = {id_: [] for id_ in ids}
-        missing_ids = set(ids)
-
-        # Mind the order, revision is the most likely one for a given ID,
-        # so we check revisions first.
-        queries: List[Tuple[str, Callable[[List[Sha1Git]], Iterable[Sha1Git]]]] = [
-            ("revision", self._cql_runner.revision_missing),
-            ("release", self._cql_runner.release_missing),
-            ("content", self.content_missing_per_sha1_git),
-            ("directory", self._cql_runner.directory_missing),
-        ]
-
-        for (object_type, query_fn) in queries:
-            found_ids = missing_ids - set(query_fn(list(missing_ids)))
-            for sha1_git in found_ids:
-                results[sha1_git].append(
-                    {
-                        "sha1_git": sha1_git,
-                        "type": object_type,
-                    }
-                )
-                missing_ids.remove(sha1_git)
-
-            if not missing_ids:
-                # We found everything, skipping the next queries.
-                break
-
-        return results
+    ##########################
+    # Origin
+    ##########################
 
     def origin_get(self, origins: List[str]) -> Iterable[Optional[Origin]]:
         return [self.origin_get_one(origin) for origin in origins]
@@ -1122,6 +1257,10 @@ class CassandraStorage:
             )
         return {"origin:add": len(origins)}
 
+    ##########################
+    # OriginVisit and OriginVisitStatus
+    ##########################
+
     def origin_visit_add(self, visits: List[OriginVisit]) -> Iterable[OriginVisit]:
         for visit in visits:
             origin = self.origin_get_one(visit.origin)
@@ -1432,6 +1571,40 @@ class CassandraStorage:
                 return visit_status
         return None
 
+    ##########################
+    # misc.
+    ##########################
+
+    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
+        results: Dict[Sha1Git, List[Dict]] = {id_: [] for id_ in ids}
+        missing_ids = set(ids)
+
+        # Mind the order, revision is the most likely one for a given ID,
+        # so we check revisions first.
+        queries: List[Tuple[str, Callable[[List[Sha1Git]], Iterable[Sha1Git]]]] = [
+            ("revision", self._cql_runner.revision_missing),
+            ("release", self._cql_runner.release_missing),
+            ("content", self.content_missing_per_sha1_git),
+            ("directory", self._cql_runner.directory_missing),
+        ]
+
+        for (object_type, query_fn) in queries:
+            found_ids = missing_ids - set(query_fn(list(missing_ids)))
+            for sha1_git in found_ids:
+                results[sha1_git].append(
+                    {
+                        "sha1_git": sha1_git,
+                        "type": object_type,
+                    }
+                )
+                missing_ids.remove(sha1_git)
+
+            if not missing_ids:
+                # We found everything, skipping the next queries.
+                break
+
+        return results
+
     def stat_counters(self):
         rows = self._cql_runner.stat_counters()
         keys = (
@@ -1451,6 +1624,10 @@ class CassandraStorage:
     def refresh_stat_counters(self):
         pass
 
+    ##########################
+    # RawExtrinsicMetadata
+    ##########################
+
     def raw_extrinsic_metadata_add(
         self, metadata: List[RawExtrinsicMetadata]
     ) -> Dict[str, int]:
@@ -1460,15 +1637,11 @@ class CassandraStorage:
             if not self._cql_runner.metadata_authority_get(
                 metadata_entry.authority.type.value, metadata_entry.authority.url
             ):
-                raise StorageArgumentException(
-                    f"Unknown authority {metadata_entry.authority}"
-                )
+                raise UnknownMetadataAuthority(str(metadata_entry.authority))
             if not self._cql_runner.metadata_fetcher_get(
                 metadata_entry.fetcher.name, metadata_entry.fetcher.version
             ):
-                raise StorageArgumentException(
-                    f"Unknown fetcher {metadata_entry.fetcher}"
-                )
+                raise UnknownMetadataFetcher(str(metadata_entry.fetcher))
 
             try:
                 row = RawExtrinsicMetadataRow(
@@ -1591,6 +1764,10 @@ class CassandraStorage:
 
         return list(results)
 
+    ##########################
+    # MetadataAuthority and MetadataFetcher
+    ##########################
+
     def raw_extrinsic_metadata_get_authorities(
         self, target: ExtendedSWHID
     ) -> List[MetadataAuthority]:
@@ -1651,7 +1828,9 @@ class CassandraStorage:
         else:
             return None
 
-    # ExtID tables
+    ##########################
+    # ExtID
+    ##########################
 
     def extid_add(self, ids: List[ExtID]) -> Dict[str, int]:
         if not self._allow_overwrite:
@@ -1754,7 +1933,10 @@ class CassandraStorage:
             )
         return result
 
-    # Misc
+    ##########################
+    # misc.
+    ##########################
+
     def clear_buffers(self, object_types: Sequence[str] = ()) -> None:
         """Do nothing"""
         return None
