@@ -1,11 +1,11 @@
-# Copyright (C) 2015-2022  The Software Heritage developers
+# Copyright (C) 2015-2023  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
 import datetime
 from enum import Enum
-from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, TypeVar
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, TypeVar, Union
 
 import attr
 from typing_extensions import Protocol, TypedDict, runtime_checkable
@@ -55,6 +55,17 @@ class PartialBranches(TypedDict):
     the snapshot has less than the request number of branches."""
 
 
+class HashDict(TypedDict, total=False):
+    sha1: bytes
+    sha1_git: bytes
+    sha256: bytes
+    blake2s256: bytes
+
+
+class TotalHashDict(HashDict, total=True):
+    pass
+
+
 @attr.s
 class OriginVisitWithStatuses:
     visit = attr.ib(type=OriginVisit)
@@ -81,6 +92,10 @@ class StorageInterface(Protocol):
         """Check that the storage is configured and ready to go."""
         ...
 
+    ##########################
+    # Content
+    ##########################
+
     @remote_api_endpoint("content/add")
     def content_add(self, content: List[Content]) -> Dict[str, int]:
         """Add content blobs to the storage
@@ -170,11 +185,11 @@ class StorageInterface(Protocol):
         ...
 
     @remote_api_endpoint("content/data")
-    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+    def content_get_data(self, content: Union[HashDict, Sha1]) -> Optional[bytes]:
         """Given a content identifier, returns its associated data if any.
 
         Args:
-            content: sha1 identifier
+            content: dict of hashes (or just sha1 identifier)
 
         Returns:
              raw content data (bytes)
@@ -228,7 +243,7 @@ class StorageInterface(Protocol):
 
     @remote_api_endpoint("content/missing")
     def content_missing(
-        self, contents: List[Dict[str, Any]], key_hash: str = "sha1"
+        self, contents: List[HashDict], key_hash: str = "sha1"
     ) -> Iterable[bytes]:
         """List content missing from storage
 
@@ -280,7 +295,7 @@ class StorageInterface(Protocol):
         ...
 
     @remote_api_endpoint("content/present")
-    def content_find(self, content: Dict[str, Any]) -> List[Content]:
+    def content_find(self, content: HashDict) -> List[Content]:
         """Find a content hash in db.
 
         Args:
@@ -308,6 +323,10 @@ class StorageInterface(Protocol):
         """
         ...
 
+    ##########################
+    # SkippedContent
+    ##########################
+
     @remote_api_endpoint("content/skipped/add")
     def skipped_content_add(self, content: List[SkippedContent]) -> Dict[str, int]:
         """Add contents to the skipped_content list, which contains
@@ -361,6 +380,10 @@ class StorageInterface(Protocol):
         """
         ...
 
+    ##########################
+    # Directory
+    ##########################
+
     @remote_api_endpoint("directory/add")
     def directory_add(self, directories: List[Directory]) -> Dict[str, int]:
         """Add directories to the storage
@@ -485,6 +508,37 @@ class StorageInterface(Protocol):
         """
         ...
 
+    @remote_api_endpoint("directory/partition/id")
+    def directory_get_id_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Sha1Git]:
+        """Splits directories into nb_partitions, and returns all the ids and
+        raw manifests in one of these based on partition_id (which must be in
+        [0, nb_partitions-1]).
+        This does not return directory entries themselves; they should be
+        retrieved using :meth:`directory_get_entries` and
+        :meth:`directory_get_raw_manifest` instead.
+
+        There is no guarantee on how the partitioning is done, or the
+        result order.
+
+        Args:
+            partition_id: index of the partition to fetch
+            nb_partitions: total number of partitions to split into
+
+        Returns:
+            Page of the directories' sha1_git hashes.
+        """
+        ...
+
+    ##########################
+    # Revision
+    ##########################
+
     @remote_api_endpoint("revision/add")
     def revision_add(self, revisions: List[Revision]) -> Dict[str, int]:
         """Add revisions to the storage
@@ -538,6 +592,30 @@ class StorageInterface(Protocol):
         """
         ...
 
+    @remote_api_endpoint("revision/partition")
+    def revision_get_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Revision]:
+        """Splits revisions into nb_partitions, and returns one of these based on
+        partition_id (which must be in [0, nb_partitions-1])
+
+        There is no guarantee on how the partitioning is done, or the
+        result order.
+
+        Args:
+            partition_id: index of the partition to fetch
+            nb_partitions: total number of partitions to split into
+
+        Returns:
+            Page of Revision model objects within the partition.
+
+        """
+        ...
+
     @remote_api_endpoint("revision")
     def revision_get(
         self, revision_ids: List[Sha1Git], ignore_displayname: bool = False
@@ -555,6 +633,56 @@ class StorageInterface(Protocol):
         """
         ...
 
+    @remote_api_endpoint("revision/log")
+    def revision_log(
+        self,
+        revisions: List[Sha1Git],
+        ignore_displayname: bool = False,
+        limit: Optional[int] = None,
+    ) -> Iterable[Optional[Dict[str, Any]]]:
+        """Fetch revision entry from the given root revisions.
+
+        Args:
+            revisions: array of root revisions to lookup
+            ignore_displayname: return the original author/committer's full name even if
+              it's masked by a displayname.
+            limit: limitation on the output result. Default to None.
+
+        Yields:
+            revision entries log from the given root root revisions
+
+        """
+        ...
+
+    @remote_api_endpoint("revision/shortlog")
+    def revision_shortlog(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
+        """Fetch the shortlog for the given revisions
+
+        Args:
+            revisions: list of root revisions to lookup
+            limit: depth limitation for the output
+
+        Yields:
+            a list of (id, parents) tuples
+
+        """
+        ...
+
+    @remote_api_endpoint("revision/get_random")
+    def revision_get_random(self) -> Sha1Git:
+        """Finds a random revision id.
+
+        Returns:
+            a sha1_git
+        """
+        ...
+
+    ##########################
+    # ExtID
+    ##########################
+
     @remote_api_endpoint("extid/from_extid")
     def extid_get_from_extid(
         self, id_type: str, ids: List[bytes], version: Optional[int] = None
@@ -613,51 +741,9 @@ class StorageInterface(Protocol):
         """
         ...
 
-    @remote_api_endpoint("revision/log")
-    def revision_log(
-        self,
-        revisions: List[Sha1Git],
-        ignore_displayname: bool = False,
-        limit: Optional[int] = None,
-    ) -> Iterable[Optional[Dict[str, Any]]]:
-        """Fetch revision entry from the given root revisions.
-
-        Args:
-            revisions: array of root revisions to lookup
-            ignore_displayname: return the original author/committer's full name even if
-              it's masked by a displayname.
-            limit: limitation on the output result. Default to None.
-
-        Yields:
-            revision entries log from the given root root revisions
-
-        """
-        ...
-
-    @remote_api_endpoint("revision/shortlog")
-    def revision_shortlog(
-        self, revisions: List[Sha1Git], limit: Optional[int] = None
-    ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
-        """Fetch the shortlog for the given revisions
-
-        Args:
-            revisions: list of root revisions to lookup
-            limit: depth limitation for the output
-
-        Yields:
-            a list of (id, parents) tuples
-
-        """
-        ...
-
-    @remote_api_endpoint("revision/get_random")
-    def revision_get_random(self) -> Sha1Git:
-        """Finds a random revision id.
-
-        Returns:
-            a sha1_git
-        """
-        ...
+    ##########################
+    # Release
+    ##########################
 
     @remote_api_endpoint("release/add")
     def release_add(self, releases: List[Release]) -> Dict[str, int]:
@@ -728,6 +814,34 @@ class StorageInterface(Protocol):
         """
         ...
 
+    @remote_api_endpoint("release/partition")
+    def release_get_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Release]:
+        """Splits releases into nb_partitions, and returns one of these based on
+        partition_id (which must be in [0, nb_partitions-1])
+
+        There is no guarantee on how the partitioning is done, or the
+        result order.
+
+        Args:
+            partition_id: index of the partition to fetch
+            nb_partitions: total number of partitions to split into
+
+        Returns:
+            Page of Release model objects within the partition.
+
+        """
+        ...
+
+    ##########################
+    # Snapshot
+    ##########################
+
     @remote_api_endpoint("snapshot/add")
     def snapshot_add(self, snapshots: List[Snapshot]) -> Dict[str, int]:
         """Add snapshots to the storage.
@@ -871,6 +985,36 @@ class StorageInterface(Protocol):
         """
         ...
 
+    @remote_api_endpoint("snapshot/partition/id")
+    def snapshot_get_id_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+    ) -> PagedResult[Sha1Git]:
+        """Splits directories into nb_partitions, and returns all the ids and
+        raw manifests in one of these based on partition_id (which must be in
+        [0, nb_partitions-1]).
+        This does not return directory entries themselves; they should be
+        retrieved using :meth:`snapshot_get_branches` instead.
+
+        There is no guarantee on how the partitioning is done, or the
+        result order.
+
+        Args:
+            partition_id: index of the partition to fetch
+            nb_partitions: total number of partitions to split into
+
+        Returns:
+            Page of the snapshots' sha1_git hashes
+        """
+        ...
+
+    ##########################
+    # OriginVisit and OriginVisitStatus
+    ##########################
+
     @remote_api_endpoint("origin/visit/add")
     def origin_visit_add(self, visits: List[OriginVisit]) -> Iterable[OriginVisit]:
         """Add visits to storage. If the visits have no id, they will be created and assigned
@@ -1099,22 +1243,9 @@ class StorageInterface(Protocol):
         """
         ...
 
-    @remote_api_endpoint("object/find_by_sha1_git")
-    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
-        """Return the objects found with the given ids.
-
-        Args:
-            ids: a generator of sha1_gits
-
-        Returns:
-            A dict from id to the list of objects found for that id. Each object
-            found is itself a dict with keys:
-
-            - sha1_git: the input id
-            - type: the type of object found
-
-        """
-        ...
+    ##########################
+    # Origin
+    ##########################
 
     @remote_api_endpoint("origin/get")
     def origin_get(self, origins: List[str]) -> Iterable[Optional[Origin]]:
@@ -1243,6 +1374,27 @@ class StorageInterface(Protocol):
         """
         ...
 
+    ##########################
+    # misc.
+    ##########################
+
+    @remote_api_endpoint("object/find_by_sha1_git")
+    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
+        """Return the objects found with the given ids.
+
+        Args:
+            ids: a generator of sha1_gits
+
+        Returns:
+            A dict from id to the list of objects found for that id. Each object
+            found is itself a dict with keys:
+
+            - sha1_git: the input id
+            - type: the type of object found
+
+        """
+        ...
+
     def stat_counters(self):
         """compute statistics about the number of tuples in various tables
 
@@ -1257,6 +1409,10 @@ class StorageInterface(Protocol):
         """Recomputes the statistics for `stat_counters`."""
         ...
 
+    ##########################
+    # RawExtrinsicMetadata
+    ##########################
+
     @remote_api_endpoint("raw_extrinsic_metadata/add")
     def raw_extrinsic_metadata_add(
         self,
@@ -1298,6 +1454,9 @@ class StorageInterface(Protocol):
         Returns:
             PagedResult of RawExtrinsicMetadata
 
+        Raises:
+            UnknownMetadataAuthority: if the metadata authority does not exist
+                at all
         """
         ...
 
@@ -1322,6 +1481,10 @@ class StorageInterface(Protocol):
         """Returns all authorities that provided metadata on the given object."""
         ...
 
+    ##########################
+    # MetadataFetcher and MetadataAuthority
+    ##########################
+
     @remote_api_endpoint("metadata_fetcher/add")
     def metadata_fetcher_add(
         self,
