@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2022  The Software Heritage developers
+# Copyright (C) 2015-2023  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -11,7 +11,20 @@ import datetime
 import itertools
 import logging
 import operator
-from typing import Any, Counter, Dict, Iterable, List, Optional, Sequence, Tuple
+from typing import (
+    Any,
+    Callable,
+    Counter,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import attr
 import psycopg2
@@ -45,9 +58,16 @@ from swh.model.model import (
     TargetType,
 )
 from swh.model.swhids import ExtendedObjectType, ExtendedSWHID, ObjectType
-from swh.storage.exc import HashCollision, StorageArgumentException, StorageDBError
+from swh.storage.exc import (
+    HashCollision,
+    StorageArgumentException,
+    StorageDBError,
+    UnknownMetadataAuthority,
+    UnknownMetadataFetcher,
+)
 from swh.storage.interface import (
     VISIT_STATUSES,
+    HashDict,
     ListOrder,
     OriginVisitWithStatuses,
     PagedResult,
@@ -105,6 +125,57 @@ def convert_validation_exceptions():
         raise StorageArgumentException(str(e))
 
 
+TRow = TypeVar("TRow", bound=Tuple)
+TResult = TypeVar("TResult")
+
+
+def _get_paginated_sha1_partition(
+    cur,
+    partition_id: int,
+    nb_partitions: int,
+    page_token: Optional[str],
+    limit: int,
+    get_range: Callable[[bytes, bytes, int, Any], Iterator[TRow]],
+    convert: Callable[[TRow], TResult],
+    get_id: Optional[Callable[[TResult], Any]] = None,
+) -> PagedResult[TResult]:
+    """Implements the bulk of ``content_get_partition``, ``directory_get_partition``,
+    ...:
+
+    1. computes range bounds
+    2. applies pagination token
+    3. converts to final objects
+    4. extracts next pagination token
+    """
+    if limit is None:
+        raise StorageArgumentException("limit should not be None")
+    if get_id is None:
+
+        def get_id(obj: TResult):
+            return obj.id  # type: ignore[attr-defined]
+
+    assert get_id is not None  # to please mypy
+
+    (start, end) = get_partition_bounds_bytes(partition_id, nb_partitions, SHA1_SIZE)
+    if page_token:
+        start = hash_to_bytes(page_token)
+    if end is None:
+        end = b"\xff" * SHA1_SIZE
+
+    next_page_token: Optional[str] = None
+    results = []
+    for counter, row in enumerate(get_range(start, end, limit + 1, cur)):
+        result = convert(row)
+        if counter >= limit:
+            # take the last content for the next page starting from this
+            next_page_token = hash_to_hex(get_id(result))
+            break
+        results.append(result)
+
+    assert len(results) <= limit
+    return PagedResult(results=results, next_page_token=next_page_token)
+
+
 class Storage:
     """SWH storage datastore proxy, encompassing DB and object storage"""
 
@@ -163,10 +234,14 @@ class Storage:
         except psycopg2.OperationalError as e:
             raise StorageDBError(e)
         self.journal_writer = JournalWriter(journal_writer)
-        self.objstorage = ObjStorage(objstorage)
+        self.objstorage = ObjStorage(self, objstorage)
         self.query_options = query_options
         self._flavor: Optional[str] = None
 
+    ##########################
+    # Utilities
+    ##########################
+
     def get_db(self):
         if self._db:
             return self._db
@@ -230,6 +305,10 @@ class Storage:
         cur.execute("select has_table_privilege(current_user, 'content', %s)", (check,))
         return cur.fetchone()[0]
 
+    ##########################
+    # Content
+    ##########################
+
     def _content_unique_key(self, hash, db):
         """Given a hash (tuple or dict), return a unique key from the
         aggregation of keys.
@@ -341,7 +420,7 @@ class Storage:
             "content:add": len(contents),
         }
 
-    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+    def content_get_data(self, content: Union[HashDict, Sha1]) -> Optional[bytes]:
         # FIXME: Make this method support slicing the `data`
         return self.objstorage.content_get(content)
 
@@ -356,29 +435,16 @@ class Storage:
         db: Db,
         cur=None,
     ) -> PagedResult[Content]:
-        if limit is None:
-            raise StorageArgumentException("limit should not be None")
-        (start, end) = get_partition_bounds_bytes(
-            partition_id, nb_partitions, SHA1_SIZE
+        return _get_paginated_sha1_partition(
+            cur,
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            db.content_get_range,
+            lambda row: Content(**dict(zip(db.content_get_metadata_keys, row))),
+            lambda row: row.sha1,
         )
-        if page_token:
-            start = hash_to_bytes(page_token)
-        if end is None:
-            end = b"\xff" * SHA1_SIZE
-
-        next_page_token: Optional[str] = None
-        contents = []
-        for counter, row in enumerate(db.content_get_range(start, end, limit + 1, cur)):
-            row_d = dict(zip(db.content_get_metadata_keys, row))
-            content = Content(**row_d)
-            if counter >= limit:
-                # take the last content for the next page starting from this
-                next_page_token = hash_to_hex(content.sha1)
-                break
-            contents.append(content)
-
-        assert len(contents) <= limit
-        return PagedResult(results=contents, next_page_token=next_page_token)
 
     @db_transaction(statement_timeout=500)
     def content_get(
@@ -403,7 +469,7 @@ class Storage:
     @db_transaction_generator()
     def content_missing(
         self,
-        contents: List[Dict[str, Any]],
+        contents: List[HashDict],
         key_hash: str = "sha1",
         *,
         db: Db,
@@ -435,9 +501,7 @@ class Storage:
             yield obj[0]
 
     @db_transaction()
-    def content_find(
-        self, content: Dict[str, Any], *, db: Db, cur=None
-    ) -> List[Content]:
+    def content_find(self, content: HashDict, *, db: Db, cur=None) -> List[Content]:
         if not set(content).intersection(DEFAULT_ALGORITHMS):
             raise StorageArgumentException(
                 "content keys must contain at least one "
@@ -461,6 +525,10 @@ class Storage:
     def content_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.content_get_random(cur)
 
+    ##########################
+    # SkippedContent
+    ##########################
+
     @staticmethod
     def _skipped_content_normalize(d):
         d = d.copy()
@@ -529,6 +597,10 @@ class Storage:
         for content in db.skipped_content_missing(contents, cur):
             yield dict(zip(db.content_hash_keys, content))
 
+    ##########################
+    # Directory
+    ##########################
+
     @db_transaction()
     def directory_add(
         self, directories: List[Directory], *, db: Db, cur=None
@@ -652,6 +724,32 @@ class Storage:
     ) -> Dict[Sha1Git, Optional[bytes]]:
         return dict(db.directory_get_raw_manifest(directory_ids, cur=cur))
 
+    @db_transaction()
+    def directory_get_id_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+        *,
+        db: Db,
+        cur=None,
+    ) -> PagedResult[Sha1Git]:
+        return _get_paginated_sha1_partition(
+            cur,
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            db.directory_get_id_range,
+            operator.itemgetter(0),
+            lambda id_: id_,
+        )
+
+    ##########################
+    # Revision
+    ##########################
+
     @db_transaction()
     def revision_add(
         self, revisions: List[Revision], *, db: Db, cur=None
@@ -709,6 +807,27 @@ class Storage:
         for obj in db.revision_missing_from_list(revisions, cur):
             yield obj[0]
 
+    @db_transaction()
+    def revision_get_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+        *,
+        db: Db,
+        cur=None,
+    ) -> PagedResult[Revision]:
+        return _get_paginated_sha1_partition(
+            cur,
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            db.revision_get_range,
+            lambda row: converters.db_to_revision(dict(zip(db.revision_get_cols, row))),
+        )
+
     @db_transaction(statement_timeout=2000)
     def revision_get(
         self,
@@ -720,7 +839,9 @@ class Storage:
     ) -> List[Optional[Revision]]:
         revisions = []
         for line in db.revision_get_from_list(revision_ids, ignore_displayname, cur):
-            revision = converters.db_to_revision(dict(zip(db.revision_get_cols, line)))
+            revision = converters.db_to_optional_revision(
+                dict(zip(db.revision_get_cols, line))
+            )
             revisions.append(revision)
 
         return revisions
@@ -754,6 +875,10 @@ class Storage:
     def revision_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.revision_get_random(cur)
 
+    ##########################
+    # ExtID
+    ##########################
+
     @db_transaction()
     def extid_get_from_extid(
         self,
@@ -821,6 +946,10 @@ class Storage:
 
         return {"extid:add": len(extid)}
 
+    ##########################
+    # Release
+    ##########################
+
     @db_transaction()
     def release_add(
         self, releases: List[Release], *, db: Db, cur=None
@@ -871,14 +1000,41 @@ class Storage:
     ) -> List[Optional[Release]]:
         rels = []
         for release in db.release_get_from_list(releases, ignore_displayname, cur):
-            data = converters.db_to_release(dict(zip(db.release_get_cols, release)))
-            rels.append(data if data else None)
+            rel = converters.db_to_optional_release(
+                dict(zip(db.release_get_cols, release))
+            )
+            rels.append(rel)
         return rels
 
+    @db_transaction()
+    def release_get_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+        *,
+        db: Db,
+        cur=None,
+    ) -> PagedResult[Release]:
+        return _get_paginated_sha1_partition(
+            cur,
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            db.release_get_range,
+            lambda row: converters.db_to_release(dict(zip(db.release_get_cols, row))),
+        )
+
     @db_transaction()
     def release_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.release_get_random(cur)
 
+    ##########################
+    # Snapshot
+    ##########################
+
     @db_transaction()
     def snapshot_add(
         self, snapshots: List[Snapshot], *, db: Db, cur=None
@@ -939,6 +1095,28 @@ class Storage:
             "next_branch": d["next_branch"],
         }
 
+    @db_transaction()
+    def snapshot_get_id_partition(
+        self,
+        partition_id: int,
+        nb_partitions: int,
+        page_token: Optional[str] = None,
+        limit: int = 1000,
+        *,
+        db: Db,
+        cur=None,
+    ) -> PagedResult[Sha1Git]:
+        return _get_paginated_sha1_partition(
+            cur,
+            partition_id,
+            nb_partitions,
+            page_token,
+            limit,
+            db.snapshot_get_id_range,
+            operator.itemgetter(0),
+            lambda id_: id_,
+        )
+
     @db_transaction(statement_timeout=2000)
     def snapshot_count_branches(
         self,
@@ -1029,6 +1207,10 @@ class Storage:
     def snapshot_get_random(self, *, db: Db, cur=None) -> Sha1Git:
         return db.snapshot_get_random(cur)
 
+    ##########################
+    # OriginVisit and OriginVisitStatus
+    ##########################
+
     @db_transaction()
     def origin_visit_add(
         self, visits: List[OriginVisit], *, db: Db, cur=None
@@ -1355,19 +1537,9 @@ class Storage:
             return OriginVisitStatus(**row_d)
         return None
 
-    @db_transaction(statement_timeout=2000)
-    def object_find_by_sha1_git(
-        self, ids: List[Sha1Git], *, db: Db, cur=None
-    ) -> Dict[Sha1Git, List[Dict]]:
-        ret: Dict[Sha1Git, List[Dict]] = {id: [] for id in ids}
-
-        for retval in db.object_find_by_sha1_git(ids, cur=cur):
-            if retval[1]:
-                ret[retval[0]].append(
-                    dict(zip(db.object_find_by_sha1_git_cols, retval))
-                )
-
-        return ret
+    ##########################
+    # Origin
+    ##########################
 
     @db_transaction(statement_timeout=1000)
     def origin_get(
@@ -1488,6 +1660,24 @@ class Storage:
                 added += 1
         return {"origin:add": added}
 
+    ##########################
+    # misc.
+    ##########################
+
+    @db_transaction(statement_timeout=2000)
+    def object_find_by_sha1_git(
+        self, ids: List[Sha1Git], *, db: Db, cur=None
+    ) -> Dict[Sha1Git, List[Dict]]:
+        ret: Dict[Sha1Git, List[Dict]] = {id: [] for id in ids}
+
+        for retval in db.object_find_by_sha1_git(ids, cur=cur):
+            if retval[1]:
+                ret[retval[0]].append(
+                    dict(zip(db.object_find_by_sha1_git_cols, retval))
+                )
+
+        return ret
+
     @db_transaction(statement_timeout=500)
     def stat_counters(self, *, db: Db, cur=None):
         return {k: v for (k, v) in db.stat_counters()}
@@ -1513,6 +1703,10 @@ class Storage:
         for key in keys:
             cur.execute("select * from swh_update_counter(%s)", (key,))
 
+    ##########################
+    # RawExtrinsicMetadata
+    ##########################
+
     @db_transaction()
     def raw_extrinsic_metadata_add(
         self,
@@ -1573,8 +1767,9 @@ class Storage:
             after_time = after
             after_fetcher = None
 
-        authority_id = self._get_authority_id(authority, db, cur)
-        if not authority_id:
+        try:
+            authority_id = self._get_authority_id(authority, db, cur)
+        except UnknownMetadataAuthority:
             return PagedResult(
                 next_page_token=None,
                 results=[],
@@ -1629,23 +1824,9 @@ class Storage:
             for row in db.raw_extrinsic_metadata_get_by_ids(ids)
         ]
 
-    @db_transaction()
-    def raw_extrinsic_metadata_get_authorities(
-        self,
-        target: ExtendedSWHID,
-        *,
-        db: Db,
-        cur=None,
-    ) -> List[MetadataAuthority]:
-        return [
-            MetadataAuthority(
-                type=MetadataAuthorityType(authority_type), url=authority_url
-            )
-            for (
-                authority_type,
-                authority_url,
-            ) in db.raw_extrinsic_metadata_get_authorities(str(target), cur)
-        ]
+    ##########################
+    # MetadataFetcher and MetadataAuthority
+    ##########################
 
     @db_transaction()
     def metadata_fetcher_add(
@@ -1668,6 +1849,24 @@ class Storage:
             return None
         return MetadataFetcher.from_dict(dict(zip(db.metadata_fetcher_cols, row)))
 
+    @db_transaction()
+    def raw_extrinsic_metadata_get_authorities(
+        self,
+        target: ExtendedSWHID,
+        *,
+        db: Db,
+        cur=None,
+    ) -> List[MetadataAuthority]:
+        return [
+            MetadataAuthority(
+                type=MetadataAuthorityType(authority_type), url=authority_url
+            )
+            for (
+                authority_type,
+                authority_url,
+            ) in db.raw_extrinsic_metadata_get_authorities(str(target), cur)
+        ]
+
     @db_transaction()
     def metadata_authority_add(
         self, authorities: List[MetadataAuthority], *, db: Db, cur=None
@@ -1701,11 +1900,11 @@ class Storage:
             authority.type.value, authority.url, cur
         )
         if not authority_id:
-            raise StorageArgumentException(f"Unknown authority {authority}")
+            raise UnknownMetadataAuthority(str(authority))
         return authority_id
 
     def _get_fetcher_id(self, fetcher: MetadataFetcher, db, cur):
         fetcher_id = db.metadata_fetcher_get_id(fetcher.name, fetcher.version, cur)
         if not fetcher_id:
-            raise StorageArgumentException(f"Unknown fetcher {fetcher}")
+            raise UnknownMetadataFetcher(str(fetcher))
         return fetcher_id
