@@ -65,8 +65,10 @@ def estimate_revision_size(revision: Revision) -> int:
     if revision.message:
         s += len(revision.message)
 
-    s += len(revision.author.fullname)
-    s += len(revision.committer.fullname)
+    if revision.author is not None:
+        s += len(revision.author.fullname)
+    if revision.committer is not None:
+        s += len(revision.committer.fullname)
     s += sum(len(h) + len(v) for h, v in revision.extra_headers)
 
     return s
@@ -132,7 +134,11 @@ class BufferingProxyStorage:
         if key.endswith("_add"):
             object_type = key.rsplit("_", 1)[0]
             if object_type in OBJECT_TYPES:
-                return partial(self.object_add, object_type=object_type, keys=["id"],)
+                return partial(
+                    self.object_add,
+                    object_type=object_type,
+                    keys=["id"],
+                )
         if key == "storage":
             raise AttributeError(key)
         return getattr(self.storage, key)
