@@ -25,6 +25,7 @@ def test_storage_direct_writer(kafka_prefix: str, kafka_server, consumer: Consum
         "client_id": "kafka_writer",
         "prefix": kafka_prefix,
         "anonymize": False,
+        "auto_flush": False,
     }
     storage_config: Dict[str, Any] = {
         "cls": "pipeline",
@@ -58,6 +59,7 @@ def test_storage_direct_writer(kafka_prefix: str, kafka_server, consumer: Consum
             expected_messages += len(objs)
         else:
             assert False, obj_type
+    storage.journal_writer.journal.flush()  # type: ignore[attr-defined]
 
     existing_topics = set(
         topic
@@ -97,6 +99,7 @@ def test_storage_direct_writer_anonymized(
         "client_id": "kafka_writer",
         "prefix": kafka_prefix,
         "anonymize": True,
+        "auto_flush": False,
     }
     storage_config: Dict[str, Any] = {
         "cls": "pipeline",
@@ -117,6 +120,7 @@ def test_storage_direct_writer_anonymized(
         method = getattr(storage, obj_type + "_add")
         method(objs)
         expected_messages += len(objs)
+    storage.journal_writer.journal.flush()  # type: ignore[attr-defined]
 
     existing_topics = set(
         topic
