@@ -70,6 +70,7 @@ def replayer_storage_and_client(
         "brokers": [kafka_server],
         "client_id": "kafka_writer",
         "prefix": kafka_prefix,
+        "auto_flush": False,
     }
     storage_config: Dict[str, Any] = {
         "cls": "memory",
@@ -106,6 +107,7 @@ def test_storage_replayer(replayer_storage_and_client, caplog):
         method = getattr(src, object_type + "_add")
         method(objects)
         nb_sent += len(objects)
+    src.journal_writer.journal.flush()
 
     caplog.set_level(logging.ERROR, "swh.journal.replay")
 
@@ -145,6 +147,7 @@ def test_storage_replay_with_collision(replayer_storage_and_client, caplog):
         method = getattr(src, object_type + "_add")
         method(objects)
         nb_sent += len(objects)
+    src.journal_writer.journal.flush()
 
     # Create collision in input data
     # These should not be written in the destination
@@ -341,6 +344,7 @@ def test_storage_replay_anonymized(
         "client_id": "kafka_writer",
         "prefix": kafka_prefix,
         "anonymize": True,
+        "auto_flush": False,
     }
     src_config: Dict[str, Any] = {"cls": "memory", "journal_writer": writer_config}
 
@@ -355,6 +359,7 @@ def test_storage_replay_anonymized(
         method = getattr(storage, obj_type + "_add")
         method(objs)
         nb_sent += len(objs)
+    storage.journal_writer.journal.flush()  # type: ignore[attr-defined]
 
     # Fill a destination storage from Kafka, potentially using privileged topics
     dst_storage = get_storage(cls="memory")
@@ -404,6 +409,7 @@ def test_storage_replayer_with_validation_ok(
         method = getattr(src, object_type + "_add")
         method(objects)
         nb_sent += len(objects)
+    src.journal_writer.journal.flush()
 
     # Fill the destination storage from Kafka
     dst = get_storage(cls="memory")
@@ -454,6 +460,20 @@ def test_storage_replayer_with_validation_nok(
         method = getattr(src, object_type + "_add")
         method([attr.evolve(TEST_OBJECTS[object_type][0], id=b"\x00" * 20)])
         nb_sent += 1
+    # also add an object that won't even be possible to instantiate; this needs
+    # to be done at low kafka level (since we cannot instantiate the invalid model
+    # object...)
+    # we use directory[1] because it actually have some entries
+    dict_repr = {
+        # copy each dir entry twice
+        "entries": TEST_OBJECTS["directory"][1].to_dict()["entries"] * 2,
+        "id": b"\x01" * 20,
+    }
+    topic = f"{src.journal_writer.journal._prefix}.directory"
+    src.journal_writer.journal.send(topic, dict_repr["id"], dict_repr)
+    nb_sent += 1
+
+    src.journal_writer.journal.flush()
 
     # Fill the destination storage from Kafka
     dst = get_storage(cls="memory")
@@ -470,7 +490,7 @@ def test_storage_replayer_with_validation_nok(
     assert invalid == 4, "Invalid objects should be detected"
     assert set(redisdb.keys()) == {
         f"swh:1:{typ}:{'0'*40}".encode() for typ in ("rel", "rev", "snp", "dir")
-    }
+    } | {b"directory:" + b"01" * 20}
 
     for key in redisdb.keys():
         # check the stored value looks right
@@ -478,7 +498,7 @@ def test_storage_replayer_with_validation_nok(
         value = kafka_to_value(rawvalue)
         assert isinstance(value, dict)
         assert "id" in value
-        assert value["id"] == b"\x00" * 20
+        assert value["id"] in (b"\x00" * 20, b"\x01" * 20)
 
     # check that invalid objects did not reach the dst storage
     for attr_ in (
@@ -488,7 +508,29 @@ def test_storage_replayer_with_validation_nok(
         "snapshots",
     ):
         for id, obj in sorted(getattr(dst._cql_runner, f"_{attr_}").iter_all()):
-            assert id != b"\x00" * 20
+            assert id not in (b"\x00" * 20, b"\x01" * 20)
+
+    # check that valid objects did reach the dst storage
+    # revisions
+    expected = [attr.evolve(rev, metadata=None) for rev in TEST_OBJECTS["revision"]]
+    result = dst.revision_get([obj.id for obj in TEST_OBJECTS["revision"]])
+    assert result == expected
+    # releases
+    expected = TEST_OBJECTS["release"]
+    result = dst.release_get([obj.id for obj in TEST_OBJECTS["release"]])
+    assert result == expected
+    # snapshot
+    # result from snapshot_get is paginated, so adapt the expected to be comparable
+    expected = [
+        {"next_branch": None, **obj.to_dict()} for obj in TEST_OBJECTS["snapshot"]
+    ]
+    result = [dst.snapshot_get(obj.id) for obj in TEST_OBJECTS["snapshot"]]
+    assert result == expected
+    # directories
+    for directory in TEST_OBJECTS["directory"]:
+        assert set(dst.directory_get_entries(directory.id).results) == set(
+            directory.entries
+        )
 
 
 def test_storage_replayer_with_validation_nok_raises(
